#!/usr/bin/env python
# -*- coding: utf-8 -*-

from optparse import OptionParser
import ConfigParser
import subprocess
import os
import time
import re
import logging

class Config(object):
    '''
    Предоставляет настройки из ini файла
    '''
    def __init__(self):
        self.config = ConfigParser.RawConfigParser()
        self.config.read("/linuxcash/cash/conf/graphicsmode.ini")

    def getBool(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.getboolean(section, option)
        except:
            pass
        return value

    def getString(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.get(section, option)
        except:
            pass
        return value
    
    def sections(self):
        return self.config.sections()

class XRandr(object):
    '''
    Взаимодействие с xrandr
    '''
    def __init__(self):
        pass

    def getAllOutputs(self):
        '''
        Все подключенные мониторы
        '''
        out = self.run(["-q"])
        return re.findall('^(.+)\sconnected', out, re.M)

    def query(self):
        '''
        Вывод информации о мониторах в консоль
        '''
        pgrep = subprocess.Popen(['pgrep', 'X'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = pgrep.communicate()
        if pgrep.returncode != 0:
            # если X сервер не запущен, запустим сами
            xServer = subprocess.Popen(['sudo', '-u', 'autologon', 'X'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            time.sleep(1)
            if xServer.poll() != None:
                out, err = xServer.communicate()
                print u"Ошибка старта X сервера:" 
                print err
                exit(1)
            print self.run(["-q"])
            xServer.terminate()
        else:
            print self.run(["-q"])

    def getModes(self):
        """
        Функция получения списка режимов, поддерживаемых одновременно всеми интерфейсами (пересечение множеств) 
        """
        ifaces = self.getInterfacesModes()

        interfaces = list()
        modes = set()
        for key in ifaces:
            if key == 'default':
                interfaces.append('default_')
            else:
                interfaces.append(key)
            if not modes:
                modes = set(ifaces[key])
            else:
                modes &= set(ifaces[key])
        modes = sorted(list(modes), key=lambda s:int(s.split('x')[0]), reverse=True)
        return modes, interfaces

    def getModesByInterface(self, interface):
        """
        Функция получения режимов для интерфейса interface
        """
        ifaces = self.getInterfacesModes()
        return ifaces[interface]

    def getInterfacesModes(self):
        """
        Функция получения всех подключеных интерфейсов и их режимов
        """
        out = self.run(['-q'])
        out = re.findall(r".*\n", out)

        ifaces = {}
        iface = ""
        for o in out:
            t = re.findall(r"^(.+)\sconnected", o)
            if t:
                iface = t[0]
                ifaces[t[0]] = []
            else:
                t = re.findall(r"   (\d+x\d+)", o)
                if t:
                    ifaces[iface].append(t[0])

        return ifaces

    def run(self, args):
        '''
        Запуск XRandr c параметрами
        '''
        args = ['sudo', '-u', 'autologon', 'xrandr'] + args
        os.environ["DISPLAY"] = ":0"
        os.environ["XAUTHORITY"] = "/home/autologon/.Xauthority"
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        if p.returncode != 0:
            logging.error(str(err))
            raise Exception, "Ошибка выполнения xrandr %s:" % err
        return out

def optionsToArgList(config, section, options):
    '''
    Считывает все опции options из секции section из настроек
    '''
    result = []
    for option in options:
        value = config.getString(section, option, None)
        if value != None:
            result.append("--" + option)
            result.append(value)
    return result

def gmMain():
    logging.basicConfig(filename='/linuxcash/logs/current/graphicsmode.log',level=logging.DEBUG, format='%(asctime)s - %(levelname)s - %(message)s')
    parser = OptionParser()
    parser.add_option("-q", "--query", action="store_true", dest="query", help=u"Вывести в консоль информацию о подключенных мониторах")
    (options, args) = parser.parse_args()

    xRandr = XRandr()
    if options.query:
        xRandr.query()
        exit(0)

    config = Config()
    if not config.getBool('main', 'setMode', False):
        logging.warning(u"Установка режима выключена в настройках")
        raise Exception, "Установка режима выключена в настройках"

    confMonitors = config.sections()
    confMonitors.remove('main')
    outputs = xRandr.getAllOutputs()

    for confMonitor in confMonitors:
        if (confMonitor not in outputs) and (confMonitor != 'default_'):
            logging.warning(u"Монитор %s заданный в настройках не подключен" % confMonitor)

    commonArgs = optionsToArgList(config, 'main', ['fb', 'dpi', 'fbmm'])
    commonMode = config.getString('main', 'mode', '800x600')

    for output in outputs:
        args = commonArgs + ['--output', output]
        if confMonitors:
            if (output in confMonitors) or (output == 'default'):
                if 'default_' in confMonitors:
                    output = 'default_'
                args.extend(['--mode', config.getString(output, 'mode', commonMode)])
                optList = ['pos', 'rate', 'reflect', 'rotate', 'set', 'scale', 'scale-from',
                           'transform', 'panning', 'gamma', 'brightness']
                args.extend(optionsToArgList(config, output, optList))
            else:
                args.append('--off')
        else:
            args.extend(['--mode', commonMode])

        logging.info(u"Установка режима: %s" % ' '.join(args))
        xRandr.run(args)

    return 0

if __name__ == "__main__":
    gmMain()